#include "board.h"

#include "gpiece.h"
#include "misc_ui.h"

#define DROP_DOWN_TIME    10
#define AFTER_GLUE_TIME   10
#define BLINK_LINE_TIME  150
#define BASE_TIME       1000

#define ORDER_TIME       200 // #### need tuning (?)
#define THINK_TIME         0

Board::Board(uint width, uint height, bool graphical, GiftPool *gp,
			 QWidget *parent, const char *name)
 : QFrame(parent, name), GenericTetris(width, height, graphical),
   _p(this), bTimer(this), tTimer(this), tBoard(0), giftPool(gp)
{
	if ( !graphical ) return;
	connect(&bTimer, SIGNAL(timeout()), SLOT(bTimeout()));
	connect(&tTimer, SIGNAL(timeout()), SLOT(tTimeout()));
	setFrameStyle( QFrame::Panel | QFrame::Sunken );
}

QSize Board::sizeHint() const
{
	int fw = frameWidth();
	return QSize(2*fw + matrix().width()*blockWidth(),
				 2*fw + matrix().height()*blockHeight());
}

QSizePolicy Board::sizePolicy() const
{
	return QSizePolicy( QSizePolicy::Fixed, QSizePolicy::Fixed );
}

void Board::setType(bool computer)
{
	if (computer) {
		if ( tBoard==0 ) tBoard = newThinkingBoard();
	} else {
		delete tBoard;
		tBoard = 0;
	}
}

void Board:: init(int seed)
{
	_seed = seed; // used here for garbage generation
	GenericTetris::init(seed);
}

void Board::start()
{
	_end        = FALSE;
	bState      = Normal;
	tState      = Idle;
	hideBoard();
	GenericTetris::start(); // NB: the timer is started by updateLevel !
}

void Board::stop()
{
	bTimer.stop();
	tTimer.stop();
}

void Board::pause()
{
	stop();
	hideBoard();
}

void Board::unpause()
{	
	showBoard();
	startBoardTimer();
	startThinkTimer(); // eventually restart thinking
}

void Board::updateRemoved(uint newNbRemoved)
{
	GenericTetris::updateRemoved(newNbRemoved);
    emit updateRemovedSignal(newNbRemoved);
}

void Board::updateScore(uint newScore)
{
	GenericTetris::updateScore(newScore);
    emit updateScoreSignal(newScore);
}

void Board::updateLevel(uint newLevel)
{
	GenericTetris::updateLevel(newLevel);
	emit updateLevelSignal(newLevel);
	startBoardTimer();
}

void Board::gluePiece()
{
	for(uint k=0; k<currentPiece()->nbBlocks(); k++) {
		int i = currentPiece()->pos(k, currentPos());
		int j = currentPiece()->line(k, currentLine());
		matrix()(i, j) = currentPiece()->value(k);
	}
	computeClearLines();
	_afterGlue();
	if ( graphical() ) startBoardTimer();
}

void Board::_afterGlue()
{
	if ( !graphical() ) afterGlue(TRUE);
	else {
		if ( afterGlue(FALSE) ) {
			bState = AfterGlue;
			return;
		}
		bState = Normal;
	}
	
	updateScore(score() + _dropHeight);
	if ( needRemoving() ) {
		loop = 0;
		beforeRemove();
		if ( graphical() ) startBoardTimer();
	} else newPiece();
}

void Board::beforeRemove()
{
	if ( graphical() ) {
		lightRemoved((loop+1)%2);
		if ( loop==0 ) bState = BeforeRemove;
		else if ( loop==3 ) bState = Normal;
		loop++;
	}

	if ( bState==Normal ) {
		remove();
		_afterRemove();
		if ( graphical() ) startBoardTimer();
	}
}

void Board::_afterRemove()
{
	if ( !graphical() ) afterRemove(TRUE);
	else {
		int r = afterRemove(FALSE);
		switch (r) {
		case 0: bState = Normal; break;
		case 1: bState = AfterRemove; return;
		case 2: 
			loop = 0;
			beforeRemove();
			return;
		}
	}

	newPiece();
}

void Board::bTimeout()
{
	switch (bState) {
	case Normal:
	case DropDown:
		if (giftPool->pending()) putGift(giftPool->take());
		_dropHeight++;
		oneLineDown();
		return;
	case BeforeRemove: beforeRemove(); break;
	case AfterGlue:    _afterGlue();   break;
	case AfterRemove:  _afterRemove(); break;
	}
	startBoardTimer();
}

void Board::startBoardTimer()
{
	switch (bState) {
	case Normal:       bTimer.start(BASE_TIME / (1 + level())); break;
	case DropDown:     bTimer.start(DROP_DOWN_TIME); break;
	case BeforeRemove: bTimer.start(BLINK_LINE_TIME, TRUE); break;
	case AfterGlue:
	case AfterRemove:  bTimer.start(AFTER_GLUE_TIME, TRUE); break;
	}
}

void Board::pDropDown()
{
	if ( bState==BeforeRemove ) return;
	_dropHeight = 0;
	oneLineDown();
	bState = DropDown;
	startBoardTimer();
}

void Board::pieceDropped(uint dropHeight)
{
	if ( bState==DropDown ) {
		bState = Normal;
		startBoardTimer();
	} else _dropHeight = dropHeight;
	
	gluePiece();
}

void Board::newPiece()
{
	GenericTetris::newPiece();
	if ( tBoard ) launchThinking(); // a human player can think by himself ...
}

/*****************************************************************************/
// AI methods
void Board::launchThinking()
{
	// clean eventual pending orders
	tTimer.stop();

	tBoard->initThinking(this);
	tState = Thinking;
	startThinkTimer();
}

void Board::initThinking(const Board *b)
{
	_nbRotations = b->currentPiece()->nbConfigurations();
	_cRotation   = 0;
	_nbPos       = matrix().width() - b->currentPiece()->width() + 1;
	_cPos        = 0;
}
	
void Board::startThinkTimer()
{
	if ( !tBoard ) return;
	switch (tState) {
	 case Idle:         break;
	 case Thinking:     tTimer.start(THINK_TIME, TRUE); break;
	 case GivingOrders: tTimer.start(ORDER_TIME, TRUE); break;
	}
}

void Board::tTimeout()
{
	switch (tState) {
	 case Idle: return;
	 case Thinking:
		if ( tBoard->think(*this, _cRotation, _cPos) ) {
			if ( _cRotation!=0 ) oneLineDown(); // so we can rotate T piece !!!
			tState = GivingOrders;
		}
		break;
	 case GivingOrders:
		if ( emitOrder() ) {
			tState = Idle;
			return;
		}
		 break;
	}

	startThinkTimer();
}

bool Board::emitOrder()
{
	if (_cRotation) {
		_cRotation--;
		rotateLeft();
	} else if ( _cPos>0 ) {
		_cPos--;
		moveRight();
	} else if ( _cPos<0 ) {
		_cPos++;
		moveLeft();
	} else {
		pDropDown();
		return TRUE; // no more orders
	}
	return FALSE;
}

/*****************************************************************************/
// Graphical methods
QRect Board::blockRect(uint i, uint j) const
{
	int fw = frameWidth();
	return QRect(fw + X(i) * blockWidth(), fw + Y(j) * blockHeight(),
				 blockWidth(), blockHeight());
}

const QColor &Board::color(const QPainter *p, int value) const
{
	switch(value) {
	 case -1: return p->backgroundColor();
	 case  0: return black;
	 default: return colors()[value-1];
	}
}

void Board::drawBlock(QPainter *p, const QRect &r, int value) const
{
	drawBlock(p, r, color(&_p, value), value<=0);
}

void Board::drawBlock(uint i, uint j, int value)
{
	drawBlock(&_p, blockRect(i, j), value);
}

void Board::paintEvent(QPaintEvent *e)
{
	_p.fillRect(e->rect(), black);
	drawFrame(&_p);
	if ( hidden() ) return;

	QRect rc = contentsRect();
	if ( e->rect().intersects(rc)==FALSE ) return; // otherwise there are
	                                               // strange bugs ...
	QRect r = e->rect().intersect(rc);
	r.moveBy(-rc.left(), -rc.top());
	updateBoard(r.left()/blockWidth(), r.top()/blockHeight(),
				r.right()/blockWidth(), r.bottom()/blockHeight());
}
