#ifndef BOARD_H
#define BOARD_H

#include <qtimer.h>
#include <qframe.h>
#include <qpainter.h>
#include "gtetris.h"

class GPieceInfo;
class GiftPool;

class Board : public QFrame, public GenericTetris
{
 Q_OBJECT

 public:
    Board(uint width, uint height, bool graphical, GiftPool *,
		  QWidget *parent, const char *name=0);
	virtual ~Board() {}

	void setType(bool computer);
	virtual void init(int seed);
	virtual void start();
	void pause();
	void unpause();
	void stop();
	
	virtual GPieceInfo *createPieceInfo() const = 0;
	
	QSize sizeHint() const;
	QSizePolicy sizePolicy() const;

	virtual uint blockWidth() const = 0;
	virtual uint blockHeight() const = 0;

	virtual uint gift() = 0;
	bool isGameOver() const { return _end; }
	const QColor &color(const QPainter *, int value) const;
	void drawBlock(QPainter *, const QRect &, int value) const;

 public slots:
    void pMoveLeft() { if ( !(bState==BeforeRemove) ) moveLeft(); }
    void pMoveRight() { if ( !(bState==BeforeRemove) ) moveRight(); }
    void pDropDown();
    void pOneLineDown() { if ( !(bState==BeforeRemove) ) oneLineDown(); }
    void pRotateLeft() { if ( !(bState==BeforeRemove) ) rotateLeft(); }
    void pRotateRight() { if ( !(bState==BeforeRemove) ) rotateRight(); }
 
 private slots:
    void bTimeout();
	void tTimeout();
    
 signals:
    void showNextPieceSignal(bool hide);
	void showPieceConfigSignal(bool hide);
    void updateRemovedSignal(int nbRemoved);
    void updateScoreSignal(int score);
    void updateLevelSignal(int level);

 protected:
	// standard methods
	void newPiece();
    void gameOver() { _end = TRUE; }
	void pieceDropped(uint dropHeight);
	virtual void gluePiece();
	virtual bool afterGlue(bool /*doAll*/)   { return FALSE; }
	virtual int afterRemove(bool /*doAll*/) { return 0; }

    void updateRemoved(uint addRemoved);
    void updateScore(uint newScore);
    void updateLevel(uint newLlevel);

	virtual void putGift(uint) = 0;
	virtual bool needRemoving() = 0;
	virtual void remove() = 0;
	
	// graphic methods
	virtual const QColor *colors() const = 0;
	QRect blockRect(uint i, uint j) const;
	void drawBlock(uint i, uint j, int value);
	virtual void drawBlock(QPainter *, const QRect &, const QColor &,
						   bool remove) const =0;
    virtual void lightRemoved(bool light) = 0;
	virtual void paintEvent(QPaintEvent *);
	
	// AI methods
	virtual bool think(const Board &, uint &nbRotations, int &decPos) = 0;
	void initThinking(const Board *);
	virtual int thinkingPoints() = 0;
	virtual Board *newThinkingBoard() = 0;
    
	QPainter _p;
	uint loop;
	int  _seed;
	uint _cRotation, _nbPos, _nbRotations;
	int  _cPos;
    
 private:
	void _afterGlue();
	void beforeRemove();
	void _afterRemove();
	void startBoardTimer();
	void startThinkTimer();
	void launchThinking();
	bool emitOrder();
	void showNextPiece(bool hide) { emit showNextPieceSignal(hide); }
	void showPieceConfig(bool hide) { emit showPieceConfigSignal(hide); }

	enum BoardState { Normal, DropDown, BeforeRemove, AfterGlue, AfterRemove };
	BoardState bState;
	QTimer     bTimer;
	uint       _dropHeight;
	bool       _end;
	QArray<QColor *> _colors;
	
	QTimer     tTimer;
	Board      *tBoard;
	enum       ThinkState { Idle, Thinking, GivingOrders };
	ThinkState tState;
	GiftPool   *giftPool;
};

#endif
