#include "fe_board.h"

#include <math.h>

#define FE_WIDTH  6
#define FE_HEIGHT 15

FEBoard::FEBoard(bool graphical, GiftPool *gp,
				 QWidget *parent, const char *name )
: Board(FE_WIDTH, FE_HEIGHT, graphical, gp, parent, name),
  groups(matrix().width(), matrix().height()),
  codes(matrix().width(), matrix().height())
{}

void FEBoard::init(int seed)
{
	Board::init(seed);
	codes.fill(0);
	nbPuyos = 0;
	chained = 0;
}

bool FEBoard::afterGlue(bool doAll)
{
	bool fragmentation = FALSE;
	for (uint j=1; j<firstClearLine(); j++) // line 0 cannot fall ...
		for (uint i=0; i<matrix().width(); i++) {
		    if ( matrix()(i, j)==0 || matrix()(i, j-1)!=0 ) continue;
			if ( j>=2 && matrix()(i, j-2)==0 ) fragmentation = TRUE;
			setAndDrawBlock(i, j-1, matrix()(i, j));
			codes(i, j-1) = 0;
			setAndDrawBlock(i, j, 0);
			codes(i, j) = 0;
		}
	specialDraw(FALSE);
	computeClearLines();
	
	if (doAll && fragmentation) afterGlue(TRUE);
	return fragmentation;
}

void FEBoard::remove()
{
	for (uint j=0; j<firstClearLine(); j++)
		for (uint i=0; i<matrix().width(); i++)
		    if ( groups(i, j)>=4 ) {
				setAndDrawBlock(i, j, 0);
				codes(i, j) = 0;
			}
	
    // score calculation from another puyo game
	// not sure it is the "official" method
	uint _nbPuyos = rem.size();
	uint _puyos = 0;
	for (uint k=0; k<rem.size(); k++) _puyos += rem[k];

	nbPuyos += _nbPuyos;
	updateRemoved(nbRemoved() + _puyos);
	chained ++;

	uint base  = 10 * _nbPuyos;
	uint bonus = _puyos - 3;
	if ( _puyos==11 ) bonus += 2;
	if ( _nbPuyos>1 ) bonus += 3*(1 << (_nbPuyos-2));
	if ( chained>1  ) bonus += 1 << (chained+1);
	uint dscore = base*bonus;
	giftRest += dscore;
	updateScore(score() + dscore);

	// update level
	if ( (nbRemoved()/100)!=((nbRemoved()-_puyos)/100) )
		updateLevel(level()+1);
}

int FEBoard::afterRemove(bool doAll)
{
	if ( afterGlue(doAll) && !doAll ) return 1;
	if ( needRemoving() ) return 2;

	chained  = 0;
	return 0;
}

#define WHITE FE_NB_COLORS

bool FEBoard::needRemoving()
{
	groups.fill(0);
	rem.resize(0);
	for (uint j=0; j<firstClearLine(); j++)
		for (uint i=0; i<matrix().width(); i++) {
			if ( matrix()(i, j)==0 || matrix()(i, j)==WHITE ) continue;
			if ( groups(i, j)!=0 ) continue;
			uint v = matrix()(i, j);
			uint nb = 1;
			treatGroup(i, j, v, nb, FALSE);
			treatGroup(i, j, v, nb, TRUE);
			if ( nb>=4 ) {
				uint s = rem.size();
				rem.resize(s+1);
				rem[s] = nb;
			}
		}
	return rem.size();
}

void FEBoard::checkBlock(uint i, uint j, uint v, uint &nb, bool set)
{
	if ( matrix()(i, j)!=v ) return;
	if ( groups(i, j)!=(set ? -1 : 0) ) return;
	treatGroup(i, j, v, nb, set);
	if (!set) nb++;
}

void FEBoard::treatGroup(uint i, uint j, uint v, uint &nb, bool set)
{
	groups(i, j) = (set ? (int)nb : -1);
	if ( i>=1 )                  checkBlock(i-1,   j, v, nb, set);
	if ( j>=1 )                  checkBlock(  i, j-1, v, nb, set);
	if ( i+1<matrix().width() )  checkBlock(i+1,   j, v, nb, set);
	if ( j+1<matrix().height() ) checkBlock(  i, j+1, v, nb, set);
}

/*****************************************************************************/
// Multiplayers methods
uint FEBoard::gift()
{
	uint n   = giftRest / 70;
	giftRest = giftRest % 70;
	return n;
}

void FEBoard::putGift(uint)
{
	// #### TODO : when some columns are too high (<5 free above), we cannot
    // put obstacle above.
	
	// #### TODO
}

/*****************************************************************************/
// AI methods
Board *FEBoard::newThinkingBoard()
{
	return new FEBoard(FALSE, 0, this);
}

	bool FEBoard::think(const Board &, uint &/*rotations*/, int &/*dec*/)
{
	return FALSE; // dummy
}

int FEBoard::thinkingPoints()
{
	return 0; // dummy
}

/*****************************************************************************/
// Graphic methods
uint FEBoard::drawCode(uint i, uint j) const
{
	uint r = 0;
	uint v = matrix()(i, j);
    if ( j!=matrix().height()-1 && matrix()(  i, j+1)==v ) r += 16;  // up
	if ( i!=matrix().width()-1  && matrix()(i+1,   j)==v ) r += 32;  // right
	if ( j!=0                   && matrix()(  i, j-1)==v ) r += 64;  // down
	if ( i!=0                   && matrix()(i-1,   j)==v ) r += 128; // left
	return r;
}

void FEBoard::specialDraw(bool force)
{
	for (uint j=0; j<firstClearLine(); j++)
		for (uint i=0; i<matrix().width(); i++) {
			uint v = matrix()(i, j);
			if (v==0 || v==WHITE ) continue;
			uint code = drawCode(i, j);
			if ( force || code!=codes(i, j) ) {
				codes(i, j) = code;
				const QColor &col = color(&_p, matrix()(i, j));
				QRect r = blockRect(i, j);
				drawBlock(&_p, r, black, TRUE);
				drawBlock(&_p, r, col, FALSE);
				specialDrawBlock(i, j, &_p, r, col);
			}
		}
}
 	 
void FEBoard::drawBlock(QPainter *p, const QRect &r,
						const QColor &col, bool remove) const
{
	if (remove) p->fillRect(r, col);
	else {
		double w = r.width(); // ASSERT( r.width()==r.height() );
		double d = (sqrt(2)-2./3)*w;
		QRect cr = QRect(0, 0, d, d);
		cr.moveCenter(QPoint(w/2, w/2) + r.topLeft());

		p->setBrush(col);
		p->setPen( QPen(NoPen) );
		p->drawEllipse(cr);
	}
}

void FEBoard::specialDrawBlock(uint i, uint j, QPainter *p, const QRect &r,
							   const QColor &col) const
{
	p->translate(r.topLeft().x(), r.topLeft().y());

	double w  = r.width();
	double a  = w/(3*sqrt(2));
	double ra = 2*w/3+1;
	uint code = codes(i, j);
	
	// first drawing with egg color
	p->setBrush(col);
	p->setPen( QPen(NoPen) );
	if ( code &  16 ) p->drawRect(    a,     0, w-2*a+1,       a); // up
	if ( code &  32 ) p->drawRect(w-a+1,     a,       a, w-2*a+1); // right
	if ( code &  64 ) p->drawRect(    a, w-a+1, w-2*a+1,       a); // down
	if ( code & 128 ) p->drawRect(    0,     a,       a, w-2*a+1); // left
	
	// second drawing with background color
	p->setBrush(black);
	QRect cr = QRect(0, 0, ra, ra);
	if ( (code & 16) || (code & 128) ) {
		cr.moveCenter(QPoint(0, 0));
		p->drawPie(cr, 16*-90, 16*90);
	}
	if ( (code & 32) || (code & 16) ) {
		cr.moveCenter(QPoint(w-1, 0));
		p->drawPie(cr, 16*180, 16*90);
	}
	if ( (code & 64) || (code & 32) ) {
		cr.moveCenter(QPoint(w-1, w-1));
		p->drawPie(cr, 16* 90, 16*90);
	}
	if ( (code &128) || (code & 64) ) {
		cr.moveCenter(QPoint(0, w-1));
		p->drawPie(cr, 16*  0, 16*90);
	}

	p->translate(-r.topLeft().x(), -r.topLeft().y());
}

void FEBoard::lightRemoved(bool light)
{
	QColor col = white;
	for (uint j=0; j<firstClearLine(); j++)
		for (uint i=0; i<matrix().width(); i++)
			if ( groups(i, j)>=4 ) {
				if (!light) col = color(&_p, matrix()(i, j));
				QRect r = blockRect(i, j);
				drawBlock(&_p, r, col, FALSE);
				specialDrawBlock(i, j, &_p, r, col);
			}
}

void FEBoard::paintEvent(QPaintEvent *e)
{
	Board::paintEvent(e);
	specialDraw(TRUE); // #### could be done only where necessary ...
}

