#include "gtetris.h"

#include "gpiece.h"

GenericTetris::GenericTetris(uint width, uint height, bool graphical)
: _nextPiece(0), _currentPiece(0), _graphical(graphical),
  _matrix(width, height)
{
	clear();
}

void GenericTetris::partialCopy(const GenericTetris &gt)
{
	_matrix        = gt._matrix;
	_currentLine   = gt._currentLine;
	_currentPos    = gt._currentPos;
	if ( _currentPiece==0 ) _currentPiece = createPiece();
	*_currentPiece = *gt._currentPiece;
	if ( _nextPiece==0 ) _nextPiece = createPiece();
	*_nextPiece    = *gt._nextPiece;
	_nbClearLines  = gt._nbClearLines;
}	

void GenericTetris::clear()
{
	_currentPos   = 0; 
	_currentLine  = -1;
	_nbClearLines = _matrix.height();
	_hidden       = FALSE;
	_matrix.fill(0);
}

GenericTetris::~GenericTetris()
{
	delete _currentPiece;
	delete _nextPiece;
}

void GenericTetris::init(int seed)
{
	if (_nextPiece==0) _nextPiece = createPiece();
	if (_currentPiece==0) _currentPiece = createPiece();
	_nextPiece->setRandomSeed(seed);
}

void GenericTetris::start()
{
	updateScore(0);
    updateLevel(1);
	updateRemoved(0);
	clear();
	_nextPiece->generateNext();
	newPiece();
}

void GenericTetris::dropDown()
{
	int dropHeight = moveTo(0, -_currentLine);
	if ( dropHeight>=0 ) pieceDropped(dropHeight);
}

void GenericTetris::oneLineDown()
{
	if ( moveTo(0, -1)==0 ) pieceDropped(0);
}

void GenericTetris::newPiece()
{
	hidePieceConfig();
	*_currentPiece = *_nextPiece;
	_currentLine  = _matrix.height() - 1 + _currentPiece->minY();
    _currentPos   =
		(_matrix.width() - _currentPiece->width())/2 - _currentPiece->minX();

    if ( !canPosition(_currentPos, _currentLine, _currentPiece)) {
		_currentLine = -1;
		gameOver();
	} else {
		showPieceConfig(FALSE);
		showPiece();
		hideNextPiece();
		_nextPiece->generateNext();
		showNextPiece(FALSE);
	}
}

bool GenericTetris::canPosition(uint pos, uint line, const GPiece *piece) const
{
    for(uint k=0; k<piece->nbBlocks(); k++) {
        uint i = piece->pos(k, pos);
        uint j = piece->line(k, line);
        if ( i>=_matrix.width() || j>=_matrix.height() ) 
			return FALSE; // outside
        if ( _matrix(i, j)!=0 ) return FALSE; // already something here
    }
    return TRUE;
}

int GenericTetris::moveTo(int decX, int decY)
{
	if ( _currentLine==-1 ) return -1;
	ASSERT(decX==0 || decY==0);
	int newPos  = _currentPos;
	int newLine = _currentLine;
	int dx = 0, dy = 0;
	uint n, i;

	if (decX) {
		dx = (decX<0 ? -1 : 1);
		n  = (uint)(decX<0 ? -decX : decX);
	} else {
		dy = (decY<0 ? -1 : 1);
		n  = (uint)(decY<0 ? -decY : decY);
	}

	for (i=0; i<n; i++) {
		if ( !canPosition(newPos + dx, newLine + dy, _currentPiece) ) break;
		newPos  += dx;
		newLine += dy;
	}
	if ( i!=0 ) { // piece moved
		hidePieceConfig();
		optimizedMove(newPos, newLine, _currentPiece);
		_currentPos  = newPos;
		_currentLine = newLine;
		showPieceConfig(FALSE);
	}
	return i;
}

void GenericTetris::rotate(bool left)
{
	if ( _currentLine==-1 ) return;
	GPiece *tmp = createPiece();
	*tmp = *_currentPiece;
    tmp->rotate(left);
    if ( canPosition(_currentPos, _currentLine, tmp) ) {
		hidePieceConfig();
		optimizedMove(_currentPos, _currentLine, tmp);
		*_currentPiece = *tmp;
		showPieceConfig(FALSE);
	}
	delete tmp;
}

void GenericTetris::computeClearLines()
{
	_nbClearLines = 0;
	for (uint j=_matrix.height(); j>0; j--) {
		for (uint i=0; i<_matrix.width(); i++)
			if ( _matrix(i, j-1)!=0 ) return;
		_nbClearLines++;
	}
}

// Graphic Methods
//     *-- x   painting coords
//     |
//     |
//     y
void GenericTetris::setAndDrawBlock(uint i, uint j, uint v)
{
	if ( _matrix(i, j)==v ) return;
	_matrix(i, j) = v;
	if ( graphical() ) drawBlock(i, j, v);
}

void GenericTetris::showBoard(bool hide)
{
	ASSERT( _hidden!=hide );
	_hidden = hide;
	updateBoard();
}
	
void GenericTetris::updateBoard(uint x1, uint y1, uint x2, uint y2)
{
	if ( !graphical() ) return;

	for(uint x=x1; x<=x2; x++)
		for(uint y=y1; y<=y2; y++) {
			uint v = _matrix(X(x), Y(y));
			if ( v==0 ) continue;
			drawBlock(X(x), Y(y), (_hidden ? 0 : v));
		}
	
	showPiece(x1, y1, x2, y2);
}

void GenericTetris::showPiece(uint x1, uint y1, uint x2, uint y2)
{
    if ( !graphical() || _currentLine==-1 ) return;

    for(uint k=0; k<_currentPiece->nbBlocks(); k++) {
		uint i = _currentPiece->pos(k, _currentPos);
		uint j = _currentPiece->line(k, _currentLine);
		if ( X(i)<(int)x1 || X(i)>(int)x2
			 || Y(j)<(int)y1 || Y(j)>(int)y2 ) continue;
        drawBlock(i, j, (_hidden ? 0 : _currentPiece->value(k)));
	}
}

void GenericTetris::optimizedMove(uint newPos, uint newLine,
								  const GPiece *newPiece)
{
	if ( !graphical() ) return;

	const uint nbc = newPiece->nbBlocks();
	QArray<GPiece::Block> updates(2*nbc);

    // fill update with the erased coords	
    for(uint i=0; i<nbc; i++) {
		updates[i].x     = _currentPiece->pos(i, _currentPos);
		updates[i].y     = _currentPiece->line(i, _currentLine);
		updates[i].value = 0;
    }
	
	uint j, k = nbc;
    for(uint i=0; i<nbc; i++) {
		updates[k].x     = newPiece->pos(i, newPos);
		updates[k].y     = newPiece->line(i, newLine);
		updates[k].value = newPiece->value(i);
		
		for (j=0; j<nbc; j++)
			if ( updates[j].x==updates[k].x && updates[j].y==updates[k].y ) {
				updates[j].value = (updates[k].value==_currentPiece->value(j) 
									? -1 : updates[k].value);
				break;
			}
		if ( j==nbc ) k++; // does not overlap an old one
    }
	
	// do the updating
    for (uint i=0; i<k; i++) {
		if ( updates[i].value==-1 ) continue;
		drawBlock(updates[i].x, updates[i].y, updates[i].value);
	}
}
